<?php

/**
 * @file
 * Provides Morningside Translator plugin controller.
 *
 * Translator @endlink. Note that we are using HTTP API.
 */

/**
 * Morningside translator plugin controller.
 */
class TMGMTMorningsideTranslatorPluginController extends TMGMTDefaultTranslatorPluginController {

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    if ($translator->getSetting('username') && $translator->getSetting('contact_email')) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::canTranslate().
   */
  public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job) {

    if (!parent::canTranslate($translator, $job)) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(TMGMTJob $job) {
    // Pull the source data array through the job and flatten it.s.
    $translator = $job->getTranslator();
    $jobLabel = $job->label;
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $representative_email = $this->fetchAccountManagerEmail($apiUrl, $username, $contact_email);
    $cc = $translator->settings['notification_email'];
    $sourceRemote = $translator->mapToRemoteLanguage($job->source_language);
    $targetRemote = $translator->mapToRemoteLanguage($job->target_language);
    $source_language = $this->fetchLanguageFromPlunet($translator, $sourceRemote);
    $target_language = $this->fetchLanguageFromPlunet($translator, $targetRemote);
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $body = array(
      "LanguageConnectID" => $username,
      "MatterName" => "Morningside_CMS_" . $job->tjid,
      "JobCreatorsName" => "Drupal",
      "JobCreatorsEmail" => $contact_email,
      "EmailTo" => $representative_email,
      "JobName" => "Morningside_CMS_" . $job->tjid,
      "DocumentTypes" => "HTML",
      "SourceLanguages" => $source_language,
      "TargetLanguage" => $target_language,
      "RequestType" => $job->getSetting('request_type'),
      "SpecialInstructions" => $job->getSetting('checkout_desc'),
      "RequestedDueDate" => $job->getSetting('request_due_date'),
      "DescriptionMessage" => "Description Message",
    );
    $options = array(
      'headers' => $header,
      'method' => 'POST',
      'data' => drupal_http_build_query($body),
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetRequest/Create', $options);

    $result_data = drupal_json_decode($result->data);

    switch ($result->code) {
      case 201:
        $job->submitted('Request successfully sent to Morningside.');
        $result_data['JobID'] = $job->tjid;
        unset($result_data['OredrIDs']);
        unset($result_data['OredrNos']);
        unset($result_data['QuoteIDs']);
        unset($result_data['QuoteNos']);
        $result_data['JobLabel'] = $jobLabel;
        db_insert('tmgmt_morningside_requests')->fields($result_data)->execute();

        $name = "JobID" . $job->tjid . '_' . $job->source_language . '_' . $job->target_language;
        $export = tmgmt_morningside_format_controller('html');
        $path = 'public://tmgmt_morningside/' . $result_data['RequestID'] . '/source/' . $name . '.html';
        $dirname = dirname($path);
        if (file_prepare_directory($dirname, FILE_CREATE_DIRECTORY)) {
          $file = file_save_data($export->export($job), $path);
          file_usage_add($file, 'tmgmt_file', 'tmgmt_job', $job->tjid);
        }
        file_create_url($file->uri);
        $handle = fopen($file->uri, "rb");
        $fsize = filesize($file->uri);
        $contents = fread($handle, $fsize);
        $byteArray = unpack("C*", $contents);
        foreach ($byteArray as $value) {
          $newByteArray[] = $value;
        }

        $token = $this->fetchToken($apiUrl, $username, $contact_email);
        $header = array(
          'Authorization' => 'Bearer ' . $token,
          'Content-Type' => 'application/x-www-form-urlencoded',
        );
        $body = array(
          "RequestID" => $result_data['RequestID'] ,
          "FolderType" => 2,
          "ContentLength" => $fsize,
          "FilePath" => "",
          "FileName" => $name . ".html",
          "FileContent" => $newByteArray,
        );
        $options = array(
          'headers' => $header,
          'method' => 'POST',
          'data' => drupal_http_build_query($body),
        );

        $result_file = drupal_http_request($apiUrl . '/api/ProjectFiles/UploadFile', $options);
        switch ($result_file->code) {
          case 201:
            $job->submitted('File submitted to Morningside. You can download file <a href="!link">here</a>.', array('!link' => file_create_url($file->uri)));
            $job->submitted('You can track request <a href="!Rlink">here</a>',
              array(
                '!Rlink' => '/admin/tmgmt/morningside-request-details/' . $result_data['RequestNo'],
              )
            );
            global $user;
            $from = $user->mail;
            if ($job->getSetting('request_type') == 'Order') {
              $mailContent = variable_get('Order-Requested');
            }
            else {
              $mailContent = variable_get('Quote-Requested');
            }

            $from = $this->fetchAccountManagerEmail($apiUrl, $username, $contact_email);

            $mailContent = str_replace("[User Name]", $user->name, $mailContent);
            $mailContent = str_replace("[Request Number]", $result_data['RequestID'], $mailContent);
            $mailContent = str_replace("[Requested Due Date]", $job->getSetting('request_due_date'), $mailContent);
            $mailContent = str_replace("[Source Language]", $source_language, $mailContent);
            $mailContent = str_replace("[Target Language(s)]", $target_language, $mailContent);

            $params = array(
              'body' => $mailContent,
              'subject' => 'Morningside Plugin : ' . $job->getSetting('request_type') . ' Requested',
              'headers' => array(
                'Cc' => $cc . ',' . $representative_email,
              ),
            );
            $this->sendEmailNotification('request_sent', $from, $contact_email, $params);
            break;

          case 400:
            $job->rejected('File Upload is rejected by Morningside Translator.');
            break;

          default:
            $job->rejected('File not upladed. Unknown error from Morningside Translator.');
            break;

        }
        break;

      case 400:
        watchdog(
          'tmgmt_morningside',
          'Create request failed : @placeholder_name',
          array('@placeholder_name' => $result_data['Message']),
          WATCHDOG_ERROR
        );
        $job->rejected('Request rejected by Morningside Translator.');
        break;

      default:
        watchdog(
          'tmgmt_morningside',
          'Create request failed : @placeholder_name',
          array('@placeholder_name' => $result_data['Message']),
          WATCHDOG_ERROR
        );
        $mailContent = variable_get('Problem-Processing-Request');

        $from = $this->fetchAccountManagerEmail($apiUrl, $username, $contact_email);
        $from = $user->mail;
        $mailContent = str_replace("[User Name]", $user->name, $mailContent);
        $mailContent = str_replace("[Requested Due Date]", $job->getSetting('request_due_date'), $mailContent);
        $mailContent = str_replace("[Source Language]", $source_language, $mailContent);
        $mailContent = str_replace("[Target Language(s)]", $target_language, $mailContent);

        $params = array(
          'body' => $mailContent,
          'subject' => 'Morningside Plugin : Problem processing request',
          'headers' => array(
            'Cc' => $cc . "," . $representative_email,
          ),
        );
        $this->sendEmailNotification('request_sent', $from, $contact_email, $params);
        $job->rejected('Request not created. Unknown error from Morningside Translator.');
        break;
    }
  }

  /**
   * TMGMTDefaultTranslatorPluginController::getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {

    $languages = $this->fetchLanguageFromPlunet($translator);

    return $languages;
  }

  /**
   * Function to fetch language name from Plunet.
   */
  public function fetchLanguageFromPlunet($translator, $langCode = NULL) {

    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $languages = array();

    $token = $this->fetchToken($apiUrl, $username, $contact_email);

    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetLanguage/GetActiveLanguages', $options);
    switch ($result->code) {
      case 200:
        $replacement = array(
          'aa' => 'aar',
          'af' => 'af-ZA',
          'ak' => 'aka',
          'sq' => 'sqi',
          'am' => 'am',
          'ar' => 'ara',
          'hy' => 'hy',
          'as' => 'asm',
          'az' => 'az-Latn',
          'bm' => 'bam',
          'eu' => 'eu-ES',
          'be' => 'bel',
          'bn' => 'ben',
          'bs' => 'bos',
          'bg' => 'bg-BG',
          'my' => 'mya',
          'ca' => 'cat',
          'ny' => 'nya',
          'zh-hans' => 'zh-CN',
          'zh-hant' => 'zh-TW',
          'hr' => 'hr-SH',
          'cs' => 'cs-CS',
          'da' => 'dan',
          'nl' => 'nl-NL',
          'en' => 'en-US',
          'en-gb' => 'en-US',
          'et' => 'et-EE',
          'ee' => 'ee-GH',
          'fo' => 'fao',
          'fj' => 'fj-FJ',
          'fil' => 'tgl',
          'fi' => 'fi-FI',
          'fr' => 'fr-FR',
          'ff' => 'ff-Latn-SN',
          'gl' => 'glg',
          'ka' => 'ka-GE',
          'de' => 'de-DE',
          'el' => 'el-GR',
          'kl' => 'kal',
          'gn' => 'gug',
          'gu' => 'guj',
          'ht' => 'cr-hat',
          'ha' => 'ha-Latn',
          'he' => 'he-IL',
          'hi' => 'hi-IN',
          'hu' => 'hu-HU',
          'is' => 'is-IS',
          'ig' => 'ibo',
          'id' => 'id-ID',
          'ga' => 'ga-IE',
          'it' => 'it-IT',
          'ja' => 'ja-JA',
          'jv' => 'jv-ID',
          'kn' => 'kn-IN',
          'ks' => 'kas',
          'kk' => 'kk-KZ',
          'ki' => 'ki-KE',
          'rw' => 'rw-RW',
          'rn' => 'rn-BI',
          'kg' => 'kon',
          'ko' => 'ko-KR',
          'ku' => 'kur',
          'ky' => 'ky-KG',
          'lo' => 'lao',
          'la' => 'lat',
          'lv' => 'lv-LV',
          'ln' => 'ln-CD',
          'lt' => 'lt-LT',
          'lg' => 'lg-UG',
          'mk' => 'mk-MK',
          'mg' => 'mlg',
          'ms' => 'ms-MY',
          'ml' => 'mal',
          'dv' => 'DIV',
          'mt' => 'mt-MT',
          'mr' => 'mr-IN',
          'mh' => 'mh-MH',
          'mo' => 'ro-MD',
          'mn' => 'mon',
          'ne' => 'ne-NP',
          'nd' => 'nd-ZW',
          'nb' => 'no-BO',
          'nn' => 'no-NO',
          'om' => 'om-ET',
          'os' => 'oss',
          'ps' => 'pus',
          'fa' => 'fas',
          'pl' => 'pl-PL',
          'pt-br' => 'pt-BR',
          'pt-pt' => 'pt-PT',
          'pa' => 'pan',
          'qu' => 'quz',
          'ro' => 'ro-RO',
          'ru' => 'ru-RU',
          'sm' => 'smo',
          'sa' => 'sa-IN',
          'sr' => 'sh-Latn',
          'sh' => 'sh-Latn',
          'sn' => 'sn-ZW',
          'sd' => 'sd-PK',
          'si' => 'si-LK',
          'sk' => 'sk-SK',
          'sl' => 'sl-SL',
          'so' => 'som',
          'nr' => 'nr-ZA',
          'es' => 'es-LA',
          'su' => 'su-ID',
          'sw' => 'swa',
          'sv' => 'sv-SE',
          'gsw-berne' => 'de-CH',
          'tl' => 'tgl',
          'ty' => 'ty-PF',
          'tg' => 'tg-Cyrl',
          'ta' => 'ta-IN',
          'tt' => 'tt-RU',
          'te' => 'te-IN',
          'th' => 'th-TH',
          'bo' => 'bo-CN',
          'ti' => 'ti-ET',
          'to' => 'ton',
          'ts' => 'ts-ZA',
          'tr' => 'tr-TR',
          'tk' => 'tk-TM',
          'tw' => 'tw-GH',
          'uk' => 'uk-UA',
          'ur' => 'ur-IN',
          'ug' => 'ug-CN',
          'uz' => 'uz-Latn-UZ',
          've' => 've-ZA',
          'vi' => 'vi-VI',
          'cy' => 'cy-GB',
          'wo' => 'wo-SN',
          'xh' => 'xh-ZA',
          'yi' => 'yid',
          'yo' => 'yo-NG',
          'zu' => 'zu',
        );
        $languageList = drupal_json_decode($result->data);
        if (is_null($langCode)) {
          foreach ($languageList as $language) {
            if (in_array($language['Code'], $replacement)) {
              $language['Code'];
              $languageCode = array_search($language['Code'], $replacement);
              $languages[$languageCode] = $languageCode;
            }
            else {
              $languageCode = $language['Code'];
              $languages[$languageCode] = $languageCode;
            }
          }
          return $languages;
        }
        else {
          if (array_key_exists($langCode, $replacement)) {
            $langCode = $replacement[$langCode];
          }
          foreach ($languageList as $language) {
            if ($language['Code'] == $langCode) {
              return $language['EnglishName'];
            }
          }
        }
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        watchdog(
          'tmgmt_morningside',
          'Unable to fetch languages From Plunet: <pre>@placeholder_name</pre>',
          array('@placeholder_name' => print_r($requestData['message'], TRUE)),
          WATCHDOG_ERROR
        );
    }
    return $languages;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getDefaultRemoteLanguagesMappings().
   */
  public function getDefaultRemoteLanguagesMappings() {
    return array(
      'zh-hans' => 'zh-CHS',
      'zh-hant' => 'zh-CHT',
    );
  }

  /**
   * TMGMTDefaultTranslatorPluginController::getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TMGMTTranslator $translator, $source_language) {

    $remote_languages = $this->getSupportedRemoteLanguages($translator);
    $languages = array();

    foreach ($remote_languages as $remote_language) {
      $local_language = $translator->mapToLocalLanguage($remote_language);
      $languages[$local_language] = $local_language;
    }

    // Check if the source language is available.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }

    return array();
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::hasCheckoutSettings().
   */
  public function hasCheckoutSettings(TMGMTJob $job) {
    return TRUE;
  }

  /**
   * Function for include aes library.
   */
  public function aesEncryptionLoadLibrary() {
    $library = libraries_load('aes-encryption');
    if (!$library || empty($library['loaded'])) {
      watchdog('tmgmt_morningside',
                'Failure to load AES Encryption PHP Client Library.',
                array(), WATCHDOG_CRITICAL);
      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Function for generate Token.
   */
  public function generateToken($apiUrl, $username, $contact_email) {

    $data = variable_get('encrypt_data');

    $password = variable_get('encrypt_password');

    $client_id = variable_get('client_id');

    $status = $this->aesEncryptionLoadLibrary();

    if ($status == TRUE) {
      $aes = new AesEncryption();
      $client_secret = $aes->encrypt($data, $password);

      $header = array(
        'Content-Type' => 'application/x-www-form-urlencoded',
      );

      $body = array(
        'username' => $username,
        'password' => $contact_email,
        'grant_type' => 'password',
        'client_id' => $client_id,
        'client_secret' => trim($client_secret),
      );

      $options = array(
        'headers' => $header,
        'method' => 'POST',
        'data' => drupal_http_build_query($body),
      );
      $result = drupal_http_request($apiUrl . '/token', $options);

      if ($result->code == 200) {
        $_SESSION['Web_API_data'] = drupal_json_decode($result->data);
        $_SESSION['Web_API_data']['createTimeStamp'] = time();
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }
  }

  /**
   * Function for fetch Token.
   */
  public function fetchToken($apiUrl, $username, $contact_email) {
    if (isset($_SESSION['Web_API_data'])) {
      $creationTime = $_SESSION['Web_API_data']['createTimeStamp'];
      $currentTime = time();
      $timeDiff = $currentTime - $creationTime;
      $expireIn = $_SESSION['Web_API_data']['as:access_token_expires_in'];
      $expireIn = (($expireIn - 1) * 60);

      if ($timeDiff > $expireIn) {
        // Refresh Token.
        $refreshTokenExpiry = $_SESSION['Web_API_data']['as:refresh_token_expires_in'];
        $refreshTokenExpiry = (($refreshTokenExpiry - 1) * 60);

        if ($timeDiff > $refreshTokenExpiry) {
          $this->generateToken($apiUrl, $username, $contact_email);
          return $_SESSION['Web_API_data']['access_token'];
        }
        else {

          $apiUrl = variable_get('API_URL');

          $data = variable_get('encrypt_data');

          $password = variable_get('encrypt_password');

          $client_id = variable_get('client_id');

          $status = $this->aesEncryptionLoadLibrary();

          if ($status == TRUE) {
            $aes = new AesEncryption();
            $client_secret = $aes->encrypt($data, $password);

            $header = array(
              'Content-Type' => 'application/x-www-form-urlencoded',
            );

            $body = array(
              'refresh_token' => $_SESSION['Web_API_data']['refresh_token'],
              'grant_type' => 'refresh_token',
              'client_id' => $client_id,
              'client_secret' => trim($client_secret),
            );

            $options = array(
              'headers' => $header,
              'method' => 'POST',
              'data' => drupal_http_build_query($body),
            );
            $result = drupal_http_request($apiUrl . '/token', $options);
            if ($result->code == 200) {

              $_SESSION['Web_API_data'] = drupal_json_decode($result->data);
              $_SESSION['Web_API_data']['createTimeStamp'] = time();
              return $_SESSION['Web_API_data']['access_token'];
            }
            else {
              unset($_SESSION['Web_API_data']);
              $this->generateToken($apiUrl, $username, $contact_email);
              if (isset($_SESSION['Web_API_data']['access_token'])) {
                return $_SESSION['Web_API_data']['access_token'];
              }
            }
          }
        }
      }
      else {
        return $_SESSION['Web_API_data']['access_token'];
      }
    }
    else {
      $this->generateToken($apiUrl, $username, $contact_email);
      if (isset($_SESSION['Web_API_data']['access_token'])) {
        return $_SESSION['Web_API_data']['access_token'];
      }
    }

  }

  /**
   * Function for Approve Request.
   */
  public function approveRequest($translator, $quoteno, $status_code) {
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded; charset=UTF-8',
    );
    $body = array(
      '' => $status_code,
    );

    $options = array(
      'headers' => $header,
      'method' => 'PUT',
      'data' => drupal_http_build_query($body),
    );
    $url = $apiUrl . '/api/PlunetQuote/SetQuoteStatus?quoteNo=' . $quoteno;
    $result = drupal_http_request($url, $options);
    if ($result->code == 200) {
      if ($status_code == 7) {
        $data = array(
          'quotestatus' => 'Accepted',
        );
      }
      else {
        $data = array(
          'quotestatus' => 'Rejected',
        );
      }

      db_update('tmgmt_morningside_quotes')
        ->fields($data)
        ->condition('quoteno', $quoteno, '=')
        ->execute();
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Function for update request using cron.
   */
  public function updateRequest($translator, $requestNo, $requestType = 'Quote') {
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetRequest/GetStatus?requestNo=' . $requestNo, $options);
    switch ($result->code) {
      case 200:
        $requestData = drupal_json_decode($result->data);
        db_update('tmgmt_morningside_requests')
          ->fields(array('requeststatus' => $requestData['RequestStatus']))
          ->condition('requestno', $requestNo, '=')
          ->execute();

        if ($requestData['QuoteNos']) {
          $quoteNos = explode(",", $requestData['QuoteNos']);
          foreach ($quoteNos as $quoteNo) {
            $result = db_select('tmgmt_morningside_quotes', 'mr')
              ->fields('mr', array('requestno'))
              ->condition('quoteno', $quoteNo, '=')
              ->condition('requestno', $requestNo, '=')
              ->execute()
              ->fetchAll();
            if (empty($result)) {
              $data = array(
                'quoteno' => $quoteNo,
                'requestno' => $requestNo,
                'quotestatus' => 'Quote in Progress',
              );
              db_insert('tmgmt_morningside_quotes')->fields($data)->execute();
            }
          }
        }
        if ($requestType == 'Order') {
          if ($requestData['OredrNos']) {
            $orderNos = explode(",", $requestData['OredrNos']);
            foreach ($orderNos as $orderNo) {
              $data = array(
                'orderno' => $orderNo,
                'orderstatus' => 'Order in Progress',
                'requestid' => $requestData['RequestID'],
              );
              db_insert('tmgmt_morningside_orders')->fields($data)->execute();
            }
          }
        }
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        watchdog(
          'tmgmt_morningside',
          'Update Request status failed (CRON): <pre>@placeholder_name</pre>',
          array('@placeholder_name' => print_r($requestData['message'], TRUE)),
          WATCHDOG_ERROR
        );
    }
  }

  /**
   * Function for update quote using cron.
   */
  public function updateQuote($translator, $quoteNo, $cc, $drupal_name) {
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetQuote/GetQuote?quoteNo=' . $quoteNo, $options);
    switch ($result->code) {
      case 200:
        $requestData = drupal_json_decode($result->data);
        $data = array(
          'quoteid' => $requestData['QuoteID'],
          'rate' => $requestData['Rate'],
          'currency' => $requestData['Currency'],
          'estimatedprice' => $requestData['EstimatedPrice'],
          'turnaroundtime' => $requestData['TurnaroundTime'],
          'services' => $requestData['Services'],
          'projectnotes' => $requestData['ProjectNotes'],
          'quotestatus' => $requestData['QuoteStatus'],
        );
        db_update('tmgmt_morningside_quotes')
          ->fields($data)
          ->condition('quoteno', $quoteNo, '=')
          ->execute();

        if ($requestData['OrderNo']) {
          $orderNos = explode(",", $requestData['OrderNo']);
          foreach ($orderNos as $orderNo) {
            $data = array(
              'orderno' => $orderNo,
              'quoteno' => $quoteNo,
              'orderstatus' => $requestData['QuoteStatus'],
              'requestid' => $requestData['RequestID'],

            );
            db_insert('tmgmt_morningside_orders')->fields($data)->execute();
            db_update('tmgmt_morningside_quotes')
              ->fields(array('quotestatus' => 'Changed into order'))
              ->condition('quoteno', $quoteNo, '=')
              ->execute();
            db_update('tmgmt_morningside_requests')
              ->fields(array('requeststatus' => 'Changed into order'))
              ->condition('requestid', $requestData['RequestID'], '=')
              ->execute();
          }
        }

        if ($requestData['QuoteStatus'] == 'Quote Complete') {
          $mailContent = variable_get('Quote-Ready');

          $from = $this->fetchAccountManagerEmail($apiUrl, $username, $contact_email);

          $mailContent = str_replace("[User Name]", $drupal_name, $mailContent);
          $mailContent = str_replace("[Request Number]", $requestData['RequestID'], $mailContent);

          $params = array(
            'body' => $mailContent,
            'subject' => 'Morningside Plugin : Quote Ready For Review',
            'headers' => array(
              'Cc' => $cc,
            ),
          );
          $this->sendEmailNotification('request_sent', $from, $contact_email, $params);
        }
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        watchdog(
          'tmgmt_morningside',
          'Update Quote status failed (CRON): <pre>@placeholder_name</pre>',
          array('@placeholder_name' => print_r($requestData['message'], TRUE)),
          WATCHDOG_ERROR
        );
    }
  }

  /**
   * Function for update order using cron.
   */
  public function updateOrder($translator, $orderNo) {
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetOrder/GetOrder?orderNo=' . $orderNo, $options);
    switch ($result->code) {
      case 200:
        $requestData = drupal_json_decode($result->data);
        $date = strtotime($requestData['DeliveryDate']);
        $data = array(
          'orderid' => $requestData['OrderID'],
          'invoiceamount' => $requestData['InvoiceAmount'],
          'deliverydate' => date('Y-m-d H:i:s', $date),
          'orderstatus' => $requestData['OrderStatus'],
        );
        db_update('tmgmt_morningside_orders')
          ->fields($data)
          ->condition('orderno', $orderNo, '=')
          ->execute();
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        watchdog(
          'tmgmt_morningside',
          'Update Order status failed (CRON): <pre>@placeholder_name</pre>',
          array('@placeholder_name' => print_r($requestData['message'], TRUE)),
          WATCHDOG_ERROR
        );
    }
  }

  /**
   * Function for Fetch file list from order number.
   */
  public function getFileList($translator, $orderNo) {
    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/ProjectFiles/GetFileList?folderType=12&orderNo=' . $orderNo, $options);

    switch ($result->code) {
      case 200:
        $requestData = drupal_json_decode($result->data);
        $token = $this->fetchToken($apiUrl, $username, $contact_email);
        $header = array(
          'Authorization' => 'Bearer ' . $token,
          'Content-Type' => 'application/x-www-form-urlencoded',
        );
        $options = array(
          'headers' => $header,
          'method' => 'GET',
        );
        $return = $requestData;
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        watchdog(
          'tmgmt_morningside',
          'Enable to fetch file list: <pre>@placeholder_name</pre>',
          array('@placeholder_name' => print_r($requestData['Message'], TRUE)),
          WATCHDOG_ERROR
        );
        $return = FALSE;
        break;
    }
    return $return;
  }

  /**
   * Function for download file.
   */
  public function downloadFile($translator, $orderNo, $requestId, $file) {

    $apiUrl = $translator->settings['API_URL'];
    $username = $translator->settings['username'];
    $contact_email = $translator->settings['contact_email'];
    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $downloadUrl = 'orderNo=' . $orderNo . '&folderType=12&filePathName=' . $file;
    $result = drupal_http_request($apiUrl . '/api/ProjectFiles/DownloadFile?' . $downloadUrl, $options);

    $returnFile = FALSE;

    if ($result->code == 200) {
      $requestData = drupal_json_decode($result->data);
      $text = base64_decode($requestData['FileContent']);
      $saveContent = tmgmt_morningside_fetch_content($text, $requestId);
      return $saveContent;
    }
    else {
      $requestData = drupal_json_decode($result->data);
      $msg = 'Not able to download ' . $file . ': <pre>@placeholder_name</pre>';
      watchdog(
        'tmgmt_morningside',
        $msg,
        array('@placeholder_name' => print_r($requestData['Message'], TRUE)),
        WATCHDOG_ERROR
      );
    }
    return $returnFile;
  }

  /**
   * Function for fetch user email ID.
   */
  public function fetchAccountManagerEmail($apiUrl, $username, $contact_email) {

    $token = $this->fetchToken($apiUrl, $username, $contact_email);
    $header = array(
      'Authorization' => 'Bearer ' . $token,
      'Content-Type' => 'application/x-www-form-urlencoded',
    );
    $options = array(
      'headers' => $header,
      'method' => 'GET',
    );
    $result = drupal_http_request($apiUrl . '/api/PlunetUser/GetPlunetUser?LanguageConnectID=' . $username, $options);

    $managerEmail = FALSE;

    switch ($result->code) {
      case 200:
        $requestData = drupal_json_decode($result->data);
        $managerEmail = $requestData['AccountManagersEmail'];
        break;

      default:
        $requestData = drupal_json_decode($result->data);
        $msg = 'Not Able to Fetch Account manager Email: <pre>@placeholder_name</pre>';
        watchdog(
          'tmgmt_morningside',
          $msg,
          array('@placeholder_name' => print_r($requestData['Message'], TRUE)),
          WATCHDOG_ERROR
        );
        break;
    }
    // $managerEmail = 'saurabh.patil@clariontechnologies.co.in';.
    return $managerEmail;
  }

  /**
   * Function for fetch user email ID.
   */
  public function sendEmailNotification($type, $from, $to, $params) {
    drupal_mail('tmgmt_morningside', $type, $to, language_default(), $params, $from, TRUE);
  }

}
