<?php

/**
 * @file
 * Provides Morningside Translator ui controller.
 */

/**
 * Morningside translator ui controller.
 */
class TMGMTMorningsideTranslatorUIController extends TMGMTDefaultTranslatorUIController {

  /**
   * Overrides TMGMTDefaultTranslatorUIController::pluginSettingsForm().
   */
  public function pluginSettingsForm($form, &$form_state, TMGMTTranslator $translator, $busy = FALSE) {
    $api_url = ($translator->getSetting('API_URL') != "") ? $translator->getSetting('API_URL') : variable_get('API_URL');
    $form['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Customer ID'),
      '#default_value' => $translator->getSetting('username'),
      '#required' => TRUE,
      '#element_validate' => array('tmgmt_morningside_plugin_settings_form_validate'),
      '#description' => t('Enter customer ID provided by Morningside.'),
    );
    $form['contact_email'] = array(
      '#type' => 'textfield',
      '#title' => t('Customer contact email'),
      '#default_value' => $translator->getSetting('contact_email'),
      '#required' => TRUE,
      '#element_validate' => array('tmgmt_morningside_plugin_settings_form_validate'),
      '#description' => t('Enter customer contact email provided to Morningside.'),
    );
    $form['notification_email'] = array(
      '#type' => 'textfield',
      '#title' => t('Additional Notification email'),
      '#default_value' => $translator->getSetting('notification_email'),
      '#description' => t('Multiple email addresses may be added, separated by comma'),
    );
    $form['API_URL'] = array(
      '#type' => 'textfield',
      '#title' => t('Morningside Connection URL'),
      '#default_value' => $api_url,
      '#description' => t('Note: Do not change this without instructions of Morningside Representative'),
    );
    return parent::pluginSettingsForm($form, $form_state, $translator);
  }

  /**
   * {@inheritdoc}
   */
  public function checkoutSettingsForm($form, &$form_state, TMGMTJob $job) {
    $form['checkout_desc'] = array(
      '#type' => 'textarea',
      '#title' => t('Project Instructions'),
      '#default_value' => $job->getSetting('checkout_desc'),
      '#description' => t('Enter any project instructions as applicable.'),
    );
    $form['type_options'] = array(
      '#type' => 'value',
      '#value' => array(
        'Quote' => t('Quote'),
        'Order' => t('Order'),
      ),
    );
    $form['request_type'] = array(
      '#type' => 'select',
      '#title' => t('Request Type'),
      '#default_value' => $job->getSetting('request_type'),
      '#description' => t('Select request type.'),
      '#options' => $form['type_options']['#value'],
    );
    $form['request_due_date'] = array(
      '#type' => 'date_popup',
      '#date_format' => 'Y-m-d',
      '#title' => t('Request Due Date'),
      '#date_label_position' => 'none',
      '#default_value' => $job->getSetting('request_due_date'),
    );
    return parent::checkoutSettingsForm($form, $form_state, $job);
  }

}
